#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de la entidad presupuesto. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para la gestión del presupuesto
#
class PresupuestoActividadController < ApplicationController

  before_filter :filtrado_listados_inicial, :only => [ :listado_por_actividades]

  before_filter :filtro_presupuesto_x_actividad, :only => [ :presupuesto_x_actividad, :modificar_crear_presupuesto_x_actividad, :eliminar_presupuesto_x_actividad ]

  # --
  # FILTROS
  # ++

  def filtro_presupuesto_x_actividad
    @etapa = Etapa.find( session[:presupuesto_proyectos_filtro_etapa] ) unless session[:presupuesto_proyectos_filtro_etapa] == "todas"
    @condiciones = { "presupuesto.proyecto_id" => @proyecto.id }
    @condiciones["presupuesto.etapa_id"] = @etapa.id unless session[:presupuesto_proyectos_filtro_etapa] == "todas"
    @condiciones["presupuesto.agente_id"] = session[:presupuesto_proyectos_filtro_agente] unless session[:presupuesto_proyectos_filtro_agente] == "todos"
    @condiciones["presupuesto.moneda_id"] = session[:presupuesto_proyectos_filtro_moneda] unless session[:presupuesto_proyectos_filtro_moneda] == "todas"
    @condiciones["presupuesto.partida_id"] = session[:presupuesto_proyectos_filtro_partida] unless session[:presupuesto_proyectos_filtro_partida] == "todas"
  end



  def filtrado_listados_inicial
    session[:presupuesto_proyectos_asc_desc] ||= "ASC"
    session[:presupuesto_proyectos_cadena_orden] ||= "partida.codigo"
    session[:presupuesto_proyectos_filtro_etapa] ||= "todas" #@proyecto.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.id
    session[:presupuesto_proyectos_filtro_agente] ||= "todos"
    session[:presupuesto_proyectos_filtro_moneda] ||= "todas"
    session[:presupuesto_proyectos_filtro_partida] ||= "todas"
  end




  def index
    redirect_to :action => :listado_por_actividades
  end



  def filtrado
    session[:presupuesto_proyectos_filtro_etapa] = params[:filtro][:etapa]
    session[:presupuesto_proyectos_filtro_agente] = params[:filtro][:agente]
    session[:presupuesto_proyectos_filtro_moneda] = params[:filtro][:moneda]
    session[:presupuesto_proyectos_filtro_partida] = params[:filtro][:partida]
    redirect_to :action => params[:listado]
  end

  def elementos_filtrado
    filtro_etapa = [["Todas", "todas"]] + @proyecto.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    filtro_agente = [["Todos", "todos"]]  + @proyecto.implementador.collect{ |e| [e.nombre, e.id] }
    filtro_moneda = [["Todas", "todas"]]  + @proyecto.moneda.collect{ |e| [e.nombre, e.id] }
    filtro_partida = [["Todas", "todas"]] + Partida.find(:all).collect {|p| [p.codigo_nombre(@proyecto.id), p.id]}
    @opciones_filtrado = [{:rotulo =>  _("Selecciona etapa: "), :nombre => "etapa", :opciones => filtro_etapa},
                          {:rotulo =>  _("Selecciona moneda: "), :nombre => "moneda", :opciones => filtro_moneda},
                          {:rotulo =>  _("Selecciona agente: "), :nombre => "agente", :opciones => filtro_agente},
			  {:rotulo =>  _("Selecciona partida: "), :nombre => "partida", :opciones => filtro_partida} ]

    @etapa = Etapa.find( session[:presupuesto_proyectos_filtro_etapa]) unless session[:presupuesto_proyectos_filtro_etapa] == "todas"
    @estado_filtrado = [(session[:presupuesto_proyectos_filtro_etapa] == "todas" ? "Cualquier etapa" : (@etapa.nombre + " (" + @etapa.fecha_inicio.to_s + "/" + @etapa.fecha_fin.to_s + ")")), 
                       (session[:presupuesto_proyectos_filtro_moneda] == "todas" ? "Cualquier moneda" : Moneda.find(session[:presupuesto_proyectos_filtro_moneda]).nombre), 
                       (session[:presupuesto_proyectos_filtro_agente] == "todos" ? "Cualquier agente" : Agente.find(session[:presupuesto_proyectos_filtro_agente]).nombre),
		       (session[:presupuesto_proyectos_filtro_partida] == "todas" ? "Cualquier partida" : Partida.find(session[:presupuesto_proyectos_filtro_partida]).nombre) ] 
    @resumen = {:url => {:action => :presupuesto, :controller => :resumen_proyecto, :sin_layout => true}}
  end


  def listado_por_actividades
    @actividades = @proyecto.actividad
    elementos_filtrado
  end

  def presupuesto_x_actividad
    @actividad = Actividad.find_by_id(params[:id])
    render :update do |page|
      page.replace_html params[:update], :partial => "sublistado_actividades"
    end
  end

  def datos_formulario
    @etapas = @proyecto.etapa.collect {|a|[a.nombre, a.id]}
    @partidas = Partida.all(:order => "codigo").collect {|a| [a.codigo_nombre(@proyecto.id), a.id]}
    @implementadores = @proyecto.implementador.all(:order => "nombre").collect {|a|[a.nombre, a.id]}
    @monedas = @proyecto.moneda.all(:order => "abreviatura").collect {|a| [a.abreviatura, a.id]} 
  end

  # en proyectos: prepara el formulario de edición o creación de presupuesto
  def editar_nuevo_presupuesto_x_actividad
    @pxa = @objeto = params[:id] ?  PresupuestoXActividad.find(params[:id]) : PresupuestoXActividad.new
    @presupuesto = @pxa.presupuesto ? @pxa.presupuesto : Presupuesto.new
    datos_formulario
    render :partial => "formulario_presupuesto_x_actividad"
  end



  def modificar_crear_presupuesto_x_actividad
    @actividad = Actividad.find_by_id(params[:actividad_id])
    @pxa = params[:id] ?  PresupuestoXActividad.find(params[:id]) : PresupuestoXActividad.new
    @presupuesto = @proyecto.presupuesto.find_by_concepto(params[:presupuesto][:concepto]) || Presupuesto.new
    ActiveRecord::Base.transaction do
      @presupuesto.presupuesto_x_actividad << @pxa
      @pxa.update_attributes params[:pxa]
      raise ActiveRecord::Rollback unless @pxa.errors.empty?
      @presupuesto.presupuesto_x_actividad << @pxa
      @presupuesto.importe = @presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.importe}
      @presupuesto.numero_unidades = @presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.numero_unidades}
      @presupuesto.attributes = params[:presupuesto]
      @presupuesto.proyecto_id = @proyecto.id
      @presupuesto.save
      @presupuesto.subpartida.update_attribute :proyecto_id, @proyecto.id if @presupuesto.subpartida
      raise ActiveRecord::Rollback unless @pxa.errors.empty? and @presupuesto.errors.empty?
    end
    #@pxa.destroy unless @presupuesto.errors.empty?
    render :update do |page|
      page.call("Modalbox.hide")
      page.replace_html params[:update], :partial => "sublistado_actividades"
      page.visual_effect :highlight, params[:update] , :duration => 6
      @objeto_modificado = {:errors =>  (@presupuesto.errors.empty? ? @pxa.errors : @presupuesto.errors) }
      @objeto_modificado[:url] =  {:action => :editar_nuevo_presupuesto_x_actividad, :id => @pxa.id, :actividad_id => params[:actividad_id], :update => params[:update] } if PresupuestoXActividad.exists?(@pxa.id)
      page.replace_html 'mensajes_modificacion', :inline => '<%= mensaje_objeto_modificado(@objeto_modificado) %>'
    end
  end



  def eliminar_presupuesto_x_actividad
    @actividad = Actividad.find_by_id(params[:actividad_id])
    @pxa = PresupuestoXActividad.find(params[:id])
    presupuesto = @pxa.presupuesto
    @pxa.destroy
    # recalculamos unidades e importe
    presupuesto.importe = presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.importe}
    presupuesto.numero_unidades = presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.numero_unidades}
    presupuesto.save unless presupuesto.importe == 0
    presupuesto.destroy if presupuesto.importe == 0
    render :update do |page|
      page.replace_html params[:update], :partial => "sublistado_actividades"
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@pxa, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

  def auto_complete_for_presupuesto_concepto
    @presupuesto = Presupuesto.find(:all, :conditions => ['presupuesto.concepto like ? and proyecto_id = ?', "%#{params[:search]}%", @proyecto.id])
    render :partial => "presupuesto_proyectos/auto_complete_for_presupuesto_concepto"
    #render :update do |page|
    #  page.auto_complete_result(@presupuesto, concepto) # "presupuesto_concepto_auto_complete", :inline => '<%= auto_complete_result @presupuesto, :concepto %>'
    #  page.show  "presupuesto_concepto_auto_complete"
    #  page.call("Modalbox.resizeToContent")
    #end 
  end

end
