#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# actividad de la matriz

# Gestiona el modelo actividad.
class Actividad < ActiveRecord::Base
  validate :codigo_mayusculas
  validates_uniqueness_of :codigo, :scope => :proyecto_id, :message => _("Código") +   _(" repetido.")
  validates_presence_of :codigo, :message => _("Código") + _(" no puede estar vacío.")
  validates_presence_of :descripcion, :message => _("Descripción") + _(" no puede estar vacío.")
  belongs_to :proyecto
  belongs_to :resultado
  has_many :gasto_x_actividad, :dependent => :destroy
  has_many :presupuesto_x_actividad, :dependent => :destroy
  has_many :presupuesto, :through => :presupuesto_x_actividad
  has_many :actividad_x_pais, :dependent => :destroy
  has_many :pais, :through => :actividad_x_pais
  has_many :actividad_x_etapa, :dependent => :destroy
  has_many :etapa, :through => :actividad_x_etapa
  has_many :actividad_detallada, :dependent => :destroy
  has_many :subactividad, :dependent => :destroy

  before_destroy :verificar_borrado

  after_create :crear_asociacion_pacs
  after_update :borrar_actividad_detallada, :modificar_asociacion_pacs
  after_destroy :eliminar_asociacion_pacs

  # Antes de guardar convertimos la fecha de fin para que sea el ultimo dia
  #before_save  "self.fecha_fin = (self.fecha_fin >> 1) - 1"

  # Devuelve codigo y nombre
  def codigo_nombre
    self.codigo + " " +self.descripcion
  end
  def codigo_descripcion
    self.codigo + " - " +self.descripcion
  end

  def codigo_mayusculas
    self.codigo = self.codigo.upcase
  end

  def crear_asociacion_pacs
    self.proyecto.pacs.each do |p|
      miact = self.clone
      mires = self.resultado ? p.resultado.find_by_codigo(self.resultado.codigo) : nil
      miact.resultado_id = mires.id if mires
      miact.proyecto_id = p.id
      miact.save
      miact.pais_ids = self.pais.collect{|p| p.id} if miact.errors.empty?
    end if self.proyecto.convenio?
  end

 private

  # Si alguno de as 
  def borrar_actividad_detallada
    actividad_detallada.each do |ad|
      ad.destroy unless etapa_ids.include?(ad.etapa_id)
    end
  end

  def verificar_borrado
    errors.add( "presupuesto", _("hay presupuestos")) unless self.presupuesto_x_actividad.empty?
    errors.add( "gasto", _("hay gastos")) unless self.gasto_x_actividad.empty?
    errors.add_to_base( _("Una actividad tiene que estar vacia para poder ser borrado.") ) unless errors.empty?
    return false unless errors.empty?
  end

  def modificar_asociacion_pacs
    self.proyecto.pacs.each do |p|
      #puts "Modificando la actividad " + self.codigo + " para el proyecto " + p.id.to_s
      miact = p.actividad.find_by_codigo(self.codigo_was)
      #puts "--------------> No tenemos la actividad!!!" unless miact
      if miact
        miact.attributes = self.attributes
        miact.proyecto_id = p.id
        mires = self.resultado ? p.resultado.find_by_codigo(self.resultado.codigo) : nil
        miact.resultado_id = mires.id if mires
        miact.save
        miact.pais_ids = self.pais.collect{|p| p.id} if miact.errors.empty?
      end
    end if self.proyecto.convenio?
  end

  def eliminar_asociacion_pacs
    self.proyecto.pacs.each do |p|
      Actividad.destroy_all(:proyecto_id => p.id, :codigo => self.codigo)
    end if self.proyecto.convenio?
  end
end
