#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

# ActiveResource que devuelve proyecto desde Alfresco.
class Proyecto < ActiveRecord::Base
  has_many :proyecto_x_pais, :dependent => :destroy
  has_many :pais, :through => :proyecto_x_pais
  belongs_to :agente
  belongs_to :moneda_principal, :class_name => "Moneda", :foreign_key => "moneda_id"
  belongs_to :moneda_intermedia, :class_name => "Moneda", :foreign_key => "moneda_intermedia_id"
  has_one :datos_proyecto
  has_one :espacio, :dependent => :destroy
  has_many :usuario_x_proyecto, :dependent => :destroy
  has_many :gasto_x_proyecto, :dependent => :destroy
  has_many :gasto, :through => :gasto_x_proyecto
  has_many :usuario, :through => :usuario_x_proyecto
  has_many :etapa, :dependent => :destroy
  has_many :libro_x_proyecto
  has_many :libro, :through => :libro_x_proyecto
  has_many :transferencia
  has_many :presupuesto
  has_many :definicion_dato, :through => :proyecto_x_definicion_dato
  has_many :proyecto_x_definicion_dato, :dependent => :destroy
  has_many :dato_texto
  has_many :documento, :through => :espacio
  has_many :objetivo_especifico
  has_many :resultado
  has_many :actividad
  has_many :estado, :dependent => :destroy
  has_one  :definicion_estado, :through => :estado, :conditions => ["estado.estado_actual"]
  has_many :tarea
  has_many :partida_financiacion
  has_many :proyecto_x_moneda, :dependent => :destroy
  has_many :moneda, :through => :proyecto_x_moneda, :uniq => true
  has_and_belongs_to_many :sector_poblacion,  :join_table => "proyecto_x_sector_poblacion"
  has_and_belongs_to_many :area_actuacion,  :join_table => "proyecto_x_area_actuacion"
  has_and_belongs_to_many :sector_intervencion,  :join_table => "proyecto_x_sector_intervencion"

  has_many :proyecto_x_financiador, :dependent => :destroy
  has_many :proyecto_x_implementador, :dependent => :destroy
  has_many :implementador, :through => :proyecto_x_implementador, :source => :agente
  has_many :financiador, :through => :proyecto_x_financiador, :source => :agente
  

  # Para cofinanciacion de proyectos
  has_many :proyecto_x_proyecto, :dependent => :destroy
  has_many :proyecto_x_proyecto_cofinanciado, :foreign_key => 'proyecto_cofinanciador_id', :class_name => 'ProyectoXProyecto', :dependent => :destroy
  has_many :proyecto_cofinanciador, :through => :proyecto_x_proyecto, :class_name => 'Proyecto'
  has_many :proyecto_cofinanciado, :through => :proyecto_x_proyecto_cofinanciado, :class_name => 'Proyecto'

  # Para convenios
  belongs_to :convenio, :class_name => "Proyecto", :foreign_key => "convenio_id"
  has_many :pacs, :class_name => "Proyecto", :foreign_key => 'convenio_id', :order => "nombre", :dependent => :destroy

  validate :nombre_mayusculas 
  validates_uniqueness_of :nombre, :message => _("Nombre") +   _(" repetido.")
  validates_uniqueness_of :titulo, :message => _("Título") +   _(" repetido.")
  validates_presence_of :titulo, :message => _("Título") + _(" no puede estar vacío.")
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_presence_of :agente_id, :message => _("Financiador") + _(" no puede estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no puede estar vacía.")

  before_validation_on_create :copiar_datos_convenio
  after_create :crear_espacio, :copiar_relaciones_convenio
  after_save :nombre_espacio, :cambiar_monedas, :asignar_otras_financiaciones_presupuesto
  after_update :modificar_datos_pacs
  before_destroy :verificar_borrado

  def convenio?
    return self.attributes["convenio_accion"] ? true : false
  end

  # Si es un pac, copia los datos basicos del convenio antes de guardarse
  def copiar_datos_convenio
    if convenio_id
      self.agente_id = self.convenio.agente_id
      self.moneda_id = self.convenio.moneda_id
      self.moneda_intermedia_id = self.convenio.moneda_intermedia_id
    end
  end

  # Si es un pac, copia las relaciones del convenio
  def copiar_relaciones_convenio
    if convenio_id
      # Copia los usuarios asignados y sus roles
      self.convenio.usuario_x_proyecto.each { |uxp| self.usuario_x_proyecto.create(:usuario_id => uxp.usuario_id, :rol => uxp.rol) }
      # Copia los elementos del menu de relaciones
      self.convenio.proyecto_x_pais.each { |pxp| self.proyecto_x_pais.create(:pais_id => pxp.pais_id) }
      self.convenio.proyecto_x_moneda.each { |mxp| self.proyecto_x_moneda.find_or_create_by_moneda_id(mxp.moneda_id) }
      self.convenio.proyecto_x_financiador.each { |pxf| self.proyecto_x_financiador.find_or_create_by_agente_id(pxf.agente_id) }
      self.convenio.proyecto_x_implementador.each { |pxi| self.proyecto_x_implementador.find_or_create_by_agente_id(pxi.agente_id) }
      self.convenio.libro_x_proyecto.each { |lxp| self.libro_x_proyecto.find_or_create_by_libro_id(lxp.libro_id) }
      ProyectoXSectorPoblacion.all(:conditions => {:proyecto_id => self.convenio_id}).each do |pxs|
        ProyectoXSectorPoblacion.create(:proyecto_id => self.id, :sector_poblacion_id => pxs.sector_poblacion_id)
      end
      ProyectoXAreaActuacion.all(:conditions => {:proyecto_id => self.convenio_id}).each do |pxa|
        ProyectoXAreaActuacion.create(:proyecto_id => self.id, :area_actuacion_id => pxa.area_actuacion_id)
      end
      ProyectoXSectorIntervencion.all(:conditions => {:proyecto_id => self.convenio_id}).each do |pxs|
        ProyectoXSectorIntervencion.create(:proyecto_id => self.id, :sector_intervencion_id => pxs.sector_intervencion_id)
      end

      # Copia las partidas y sus relaciones
      self.convenio.partida_financiacion.each do |pf|
        #mipf = self.partida_financiacion.create(pf.attributes)
        mipf = pf.clone
        mipf.proyecto_id = self.id
        mipf.save
        pf.partida_x_partida_financiacion.each { |pxpf| mipf.partida_x_partida_financiacion.create(:partida_id => pxpf.partida_id) }
      end

      # Copia los objetivos especificos, resultados y actividades (junto a las hipotesis, fuentes e indicadores)
      self.convenio.objetivo_especifico.each do |obj|
        obj.crear_asociacion_pacs
        obj.fuente_verificacion.each{|fv| fv.crear_asociacion_pacs}
        obj.indicador.each{|ind| ind.crear_asociacion_pacs}
        obj.hipotesis.each{|hip| hip.crear_asociacion_pacs}
      end
      self.convenio.resultado.each do |obj|
        obj.crear_asociacion_pacs
        obj.fuente_verificacion.each{|fv| fv.crear_asociacion_pacs}
        obj.indicador.each{|ind| ind.crear_asociacion_pacs}
        obj.hipotesis.each{|hip| hip.crear_asociacion_pacs}
      end
      self.convenio.actividad.each do |obj|
        obj.crear_asociacion_pacs
      end
    end
  end

  # Si es un convenio, copia los datos a sus pacs despues de una modificacion
  def modificar_datos_pacs
    self.pacs.each do |pac|
      pac.agente_id = self.agente_id
      pac.moneda_id = self.moneda_id
      pac.moneda_intermedia_id = self.moneda_intermedia_id
      pac.save      
    end if self.convenio?
  end

  def crear_espacio
    Espacio.create :nombre => nombre, :proyecto_id => id, :espacio_padre_id => Espacio.find_by_nombre("Proyectos").id , :descripcion => ("Espacio raiz del proyecto: " + nombre)
  end

  def nombre_espacio
    espacio.update_attribute "nombre", nombre
  end

  def asignar_otras_financiaciones_presupuesto
    financiacion_privada = Agente.find_by_nombre("OTRAS FINANCIACIONES PRIVADAS")
    financiacion_publica = Agente.find_by_nombre("OTRAS FINANCIACIONES PÚBLICAS")
    ProyectoXFinanciador.find_or_create_by_agente_id_and_proyecto_id(financiacion_privada.id, id)
    ProyectoXFinanciador.find_or_create_by_agente_id_and_proyecto_id(financiacion_publica.id, id)
  end

  def dato_dinamico id , tipo 
    eval( "Dato" + tipo ).find(id) unless id == 0
    eval( "Dato" + tipo ).new if id == 0 
  end

  def datos_dinamicos pestana
    # ( dato_integro.select{|d| d.definicion_dato.pestana == pestana } + dato_cadena.select{|d| d.definicion_dato.pestana == pestana }  + dato_texto.select{|d| d.definicion_dato.pestana == pestana } ).sort{ |a,b| a.definicion_dato.rango <=> b.definicion_dato.rango }
    ( dato_texto.select{|d| d.definicion_dato.pestana == pestana } ).sort{ |a,b| a.definicion_dato.rango <=> b.definicion_dato.rango }
  end

  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

  def estado_actual
    (estado.select { |a|  a.estado_actual }).first
  end

 # Devuelve el presupuesto total del proyecto en la moneda base o el de la moneda que se especifique
 def presupuesto_total mon=nil, financ=nil
   total = 0.0
   financ = nil unless financ.class.name == "Agente"
   condiciones = {} 
   #fecha_inicio = self.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.fecha_inicio if fecha_fin
   #condiciones = { :fecha => fecha_inicio..fecha_fin } if fecha_fin && fecha_fin.class.name == "Date"
   condiciones["presupuesto_x_agente.agente_id"] = financ.id if financ
   condiciones["presupuesto.moneda_id"] = mon.id if mon and mon.class.name=="Moneda"
   self.presupuesto.all(:include => "presupuesto_x_agente", :conditions => condiciones).each do |pre|
     importe = financ ? pre.presupuesto_x_agente.first.importe : pre.importe
     total += importe if mon
     total += (importe * pre.tasa_cambio).round(2) unless mon
   end
   return total
 end

 # Devuelve el gasto total del proyecto en la indicada o la moneda base si no esta definida
 def gasto_total financ=nil, mon=nil, etap=nil, pai=nil
   total = 0.0
   financ = nil unless financ.class.name == "Agente"
   etap = nil unless etap.class.name == "Etapa"
   mon = nil unless mon.class.name == "Moneda"
   pai = nil unless pai.class.name == "Pais"
   condiciones = {}
   condiciones["gasto_x_agente.agente_id"] = financ.id if financ
   condiciones["gasto.fecha"] = etap.fecha_inicio..etap.fecha_fin if etap
   condiciones["gasto.moneda_id"] = mon.id if mon
   # Recorremos todos los gastos
   self.gasto.all(:include => ["gasto_x_agente"], :conditions => condiciones).each do |gas|
     # PENDIENTE: Tenemos que filtrar solo los gastos pagados con una cuenta del pais indicado (si existe)
     importe_pago = 0
     gas.pago.each do |pago|
       importe_pago += pago.importe if pago.libro && pago.libro.pais && pago.libro.pais == pai 
     end
     # Pillamos el gasto por financiador si se ha pedido eso (financ != nil)
     gxa = gas.gasto_x_agente.first if financ
     # Pillamos el gasto por proyecto (necesario para la tasa de cambio)
     gxp = gas.gasto_x_proyecto.find_by_proyecto_id self.id
     # Escogemos el importe segun sea de financiador o completo del proyecto
     importe = gxa ? gxa.importe : gxp.importe

     # Averiguamos el importe de los pagos realizados (y en el caso de que hayamos definido pais, sobre cuenta del pais)
     importe_pagado = 0
     gas.pago.each { |pago| importe_pagado += pago.importe if pai.nil? || pago.libro.pais == pai }

     # Y escogemos el adecuado (puede que no este todo pagado, o que los pagos sean mayores que el importe)
     importe = [importe, importe_pagado].min if importe >= 0
     importe = [importe, importe_pagado].max if importe < 0

     # Si no hay moneda, le aplicamos la tasa de cambio
     total += (importe * gxp.tasa_cambio).round(2) if importe && gxp && mon.nil?
     # Si hay moneda, no convertimos el importe 
     total += importe if importe && mon
   end
   #puts "----------------> Nos piden el gasto para"
   #puts "                  PAIS: " + pai.nombre if pai
   #puts "                  MONEDA: " + mon.nombre if mon
   #puts "                  ETAPA: " + etap.nombre if etap
   #puts "                  FINANC: " + financ.nombre if financ
   #puts "                  " + total.to_s
   return total
 end

 # Devuelve el total de transferencias realizadas (para una o todas las monedas)
 def transferido_total mon, etap=nil, financ=nil, direccion=nil, tipo=nil, pai=nil
   total = 0.0
   pai = nil unless pai.class.name == "Pais"
   if mon && mon.class.name == "Moneda"

     # Filtros basicos de la transferencia... luego los complicaremos sobre los resultados
     condiciones  = "moneda_id = " + mon.id.to_s
     condiciones += " AND tipo = '" + tipo.to_s + "'" if tipo
     condiciones += " AND agente_id = " + financ.id.to_s if financ.class.name == "Agente"
     condiciones += " AND fecha BETWEEN '" + etap.fecha_inicio.to_s + "' AND '" + etap.fecha_fin.to_s + "'" if etap.class.name == "Etapa"
     condiciones += " AND entrante_saliente = '" + direccion + "'" if direccion
    
     #puts "---------------------> Miramos sobre la moneda " + mon.abreviatura + " con las condiciones " + condiciones
     #puts "---------------------> Pais: " + pai.nombre if pai
     self.transferencia.all(:conditions => condiciones).each do |trans|
       # Contabiliza solo las que sean a distinta moneda o el mismo tipo de libro a no ser que sea sin segunda cuenta
       #if (tipo && trans.tipo =~ /subvencion|intereses|adelanto/) || (trans.libro_receptor_emisor && (trans.moneda_id != trans.libro_receptor_emisor.moneda_id || trans.libro.tipo == trans.libro_receptor_emisor.tipo))
       # Contabiliza solo las que sean a distinta moneda o distinto pais a no ser que sea sin segunda cuenta 
       if (tipo && trans.tipo =~ /subvencion|intereses|adelanto/) || (trans.libro_receptor_emisor && (trans.moneda_id != trans.libro_receptor_emisor.moneda_id || trans.libro.pais_id != trans.libro_receptor_emisor.pais_id))
         # Contabiliza solo las transferencias del pais (o si no se pide por pais, todas)
         if pai == nil || trans.libro.pais == pai
           #puts "        " + trans.importe.to_s + " " + trans.entrante_saliente
           #puts "        " + trans.libro.pais.nombre
           # Si se ha indicado la direccion, se contabilizan solo esas
           total += trans.importe if direccion && trans.entrante_saliente == direccion
           # Si no se indica direccion, se restan las entrantes de las salientes
           total += trans.importe if trans.entrante_saliente == "entrante" && direccion.nil?
           total -= trans.importe if trans.entrante_saliente == "saliente" && direccion.nil?
         end
       end
     end
   end
   return total
 end


  def presupuesto_cofinanciado_privado
    
  end

  private

  def cambiar_monedas
    # Se asegura de que la moneda principal este asignada
    ProyectoXMoneda.find_or_create_by_proyecto_id_and_moneda_id(self.id, self.moneda_id) if moneda_id_changed? && moneda_id
    # Y tambien la divisa
    ProyectoXMoneda.find_or_create_by_proyecto_id_and_moneda_id(self.id, self.moneda_intermedia_id) if moneda_intermedia_id_changed? && moneda_intermedia_id

    # Para modificaciones de proyecto que incluyan moneda principal, actualiza las tasas de cambio
    self.etapa.each { |e| TasaCambio.cambia_moneda(e) } if id_was && moneda_id_changed?
  end

  def verificar_borrado
    if self.datos_proyecto
      (DatosProyecto.column_names - ["id", "proyecto_id"]).each do |v|
        if not self.datos_proyecto.send( v.to_sym ).nil?
          errors.add( "datos_proyecto","hay datos de proyecto " + v.humanize )
        end
      end
    end
    errors.add( "gasto", "hay gastos" ) unless self.gasto_x_proyecto.empty?
    errors.add( "presupuesto", "hay presupuestos" ) unless self.presupuesto.empty?
    errors.add( "objetivo_especifico", "hay objetivos especificos" ) unless self.objetivo_especifico.empty?
    errors.add( "documento", "hay documentos" ) unless self.documento.empty?
    errors.add( "tarea", "hay tareas" ) unless self.tarea.empty?
    # la relacion habtm no tiene :dependent por eso tenemos que borrar las relaciones de financiador y implementador 
    #errors.add( _("esta relacionado con agentes como implementadores") ) unless self.implementador == []
    #errors.add( _("esta relacionado con agentes como financiadores") ) unless self.financiador == []
    errors.add_to_base( "Un proyecto tiene que estar vacio para poder ser borrado." ) unless errors.empty?
   return false unless errors.empty?
 end

end
