#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# resultados de la matriz

class Resultado < ActiveRecord::Base
  validate :codigo_mayusculas
  validates_uniqueness_of :codigo, :scope => :proyecto_id, :message => _("Código") +   _(" repetido.")
  validates_presence_of :descripcion, :message => _("Descripción") + _(" no puede estar vacío.")

  belongs_to :proyecto
  belongs_to :objetivo_especifico
  has_many :actividad
  has_many :hipotesis, :dependent => :destroy
  has_many :indicador, :dependent => :destroy
  has_many :fuente_verificacion, :dependent => :destroy

  # Esto no funciona (comprobar verificacion de borrado y modificacion para pacs)
  #before_save :verificar_modificacion_pac
  before_destroy :verificar_borrado
  after_create :crear_asociacion_pacs
  after_update :modificar_asociacion_pacs
  after_destroy :eliminar_asociacion_pacs

  def codigo_mayusculas
    self.codigo = self.codigo.upcase
  end

  # Devuelve codigo y nombre
  def codigo_nombre
    self.codigo + " " +self.descripcion
  end

  def crear_asociacion_pacs
    self.proyecto.pacs.each do |p|
      mires = self.clone
      mires.proyecto_id = p.id
      mires.objetivo_especifico_id = p.objetivo_especifico.find_by_codigo(self.objetivo_especifico.codigo).id
      mires.save
    end if self.proyecto.convenio?
  end

 private

  def verificar_modificacion_pac
    errors.add_to_base _("Una acción no se puede modificar desde el PAC.") if self.proyecto.convenio && self.proyecto.convenio.convenio_accion == "resultado"
    return false unless errors.empty?
  end

  def verificar_borrado
    errors.add( "actividad", _("hay actividades")) unless self.actividad.empty?
    errors.add_to_base( _("Un resultado tiene que estar vacio para poder ser borrado.") ) unless errors.empty?
    #errors.add_to_base _("Una acción no se puede borrar desde el PAC.") if self.proyecto.convenio && self.proyecto.convenio.convenio_accion == "resultado" && self.proyecto.convenio.resultado.find_by_codigo(self.codigo)
    return false unless errors.empty?
  end

  def modificar_asociacion_pacs
    self.proyecto.pacs.each do |p|
      mires = p.resultado.find_or_create_by_codigo(self.codigo_was)
      if mires
        mires.attributes = self.attributes
        mires.proyecto_id = p.id
        mires.objetivo_especifico_id = p.objetivo_especifico.find_by_codigo(self.objetivo_especifico.codigo).id
        mires.save
      end
    end if self.proyecto.convenio?
  end

  def eliminar_asociacion_pacs
    self.proyecto.pacs.each do |p|
      miobj = p.objetivo_especifico.find_by_codigo(self.objetivo_especifico.codigo)
      Resultado.destroy_all(:proyecto_id => p.id, :objetivo_especifico_id => miobj.id, :codigo => self.codigo) if miobj
    end if self.proyecto.convenio?
  end
end
