#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

# Clases del modelo que gestiona la entidad Transferencia.

class Transferencia < ActiveRecord::Base

  untranslate_all

  belongs_to :moneda
  belongs_to :libro
  belongs_to :libro_segundo, :class_name => "Libro", :foreign_key => "libro_receptor_emisor_id"
  belongs_to :etapa
  belongs_to :agente
  belongs_to :proyecto
  has_and_belongs_to_many :gasto, :join_table =>  "gasto_x_transferencia"
  has_many :gasto_x_transferencia, :dependent => :destroy
  belongs_to :transferencia_vinculada, :class_name => "Transferencia", :foreign_key => "transferencia_id"

  validates_presence_of :importe, :message => _("Importe") + _(" no puede estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no puede estar vacío.")
  validates_presence_of :libro_id, :message => _("Cuenta") + _(" no puede estar vacío.")
  validates_presence_of :entrante_saliente, :message => _("Tipo") + _(" no puede estar vacío.")
  validates_presence_of :tipo, :message => _("Tipologia de movimiento no definida")
  validates_presence_of :fecha,  :message => _("Fecha") + _(" no puede estar vacío.")

  # Mantiene vinculos entre transferencias
  before_save :comprobar_cambios
  before_save :comprobar_moneda
  before_destroy :eliminar_vinculo_relacionada
  after_create :vincular_relacionada
  # Usamos estos callbacks para invocar la regeneracion de tasas de cambio
  after_save 'TasaCambio.actualiza_ponderadas(self)'
  after_destroy 'TasaCambio.actualiza_ponderadas(self)'


  # Comprueba que la moneda sea la misma que la del libro
  def comprobar_moneda
    self.moneda_id = self.libro.moneda_id if self.libro 
  end

  # Comprueba si se ha producido un cambio en la direccion estando asociada, en cuyo caso, elimina la asociacion
  def comprobar_cambios
    desvincular = false
    if self.transferencia_vinculada
      desvincular = true if !self.new_record? && self.entrante_saliente_changed?
      desvincular = true if self.libro.moneda_id != self.transferencia_vinculada.libro_receptor_emisor.moneda_id if self.libro
      desvincular = true if self.libro_receptor_emisor.moneda_id != self.transferencia_vinculada.libro.moneda_id if self.libro_receptor_emisor
    end
    if desvincular
      self.transferencia_vinculada.update_attribute :transferencia_id, nil
      self.transferencia_id = nil
    end
  end

  # Vincula la relacionada con esta despues de crear
  def vincular_relacionada
    self.transferencia_vinculada.update_attribute(:transferencia_id, self.id) if self.transferencia_vinculada
  end

  # Elimina el vinculo con la relacionada cuando se va a eliminar
  def eliminar_vinculo_relacionada
    self.transferencia_vinculada.update_attribute(:transferencia_id, nil) if self.transferencia_vinculada 
  end

  # Comprueba que la fecha está dentro de la etapa seleccionada
  def comprobar_fecha_etapa etapa
    if self.fecha > etapa.fecha_inicio and self.fecha < etapa.fecha_fin
      return true
    else
      errors.add(_("Fecha"), _("La fecha debe estar dentro de la etapa")) ; return false
    end
  end
  # Moneda de la transferencia
  def moneda
    return self.moneda_id ? Moneda.find(self.moneda_id) : nil
  end
  # Proyecto al que se ha asignado la transferencia
  def proyecto
    return Proyecto.find(self.proyecto_id) if self.proyecto_id  
  end

  # Devuelve el tipo completo de la transferencia
  def tipo_completo
    return self.tipo.capitalize + " " + self.entrante_saliente unless self.tipo == "intereses" || self.tipo == "subvencion"
    return self.tipo.capitalize if self.tipo == "intereses" || self.tipo == "subvencion"
  end

  # Libro receptor o emisor (segun el tipo de la transferencia)
  def  libro_receptor_emisor
     return Libro.find(self.libro_receptor_emisor_id) if self.libro_receptor_emisor_id
  end

  # Moneda a la que hace referencia la tasa de cambio
  def moneda_tasa_cambio
    return "" if entrante_saliente == "saliente"
    return "(" + (libro_receptor_emisor_id ? Libro.find_by_id(libro_receptor_emisor_id).moneda.abreviatura : libro.moneda.abreviatura) + ")"
  end

  # Devuelve si la transferencia esta vinculada o no
  def vinculada?
    !(self.transferencia_vinculada.nil? && self.tipo =~ /transferencia|cambio|cheque|retirada|ingreso/)
  end
end
